
from django.contrib.auth.models import User, Group
from django.urls import reverse
from django.test import RequestFactory, TestCase
from rest_framework.test import APIClient
from uhepp_vault.models import Collection, Plot

from uhepp_vault.models import PUBLIC_LEVEL, INTERNAL_LEVEL, PRIVATE_LEVEL

class UsersEndpointTestCase(TestCase):
    """Test the /users endpoint of the api"""
    fixtures = ["users.json", "collections.json"]

    def setUp(self):
        self.client = APIClient()
        self.frank = User.objects.get(username="cern-fsauerbu")
        self.karl = User.objects.get(username="cern-karl")
        self.john = User.objects.get(username="cern-john")

    ###########################################################
    # User list
    def test_users_list_anon(self):
        """Check that private and internal profiles are not listed to anon"""
        response = self.client.get(reverse("user-list"))
        user_names = sorted(user["username"] for user in response.json())
        self.assertEqual(user_names, ["cern-fsauerbu"])

    def test_users_list_auth(self):
        """Check that private profiles are not listed to auth"""
        self.client.force_authenticate(self.frank)
        response = self.client.get(reverse("user-list"))
        user_names = sorted(user["username"] for user in response.json())
        self.assertEqual(user_names, ["cern-fsauerbu", "cern-karl"])

    def test_users_list_self_public(self):
        """Check that one's own public profile is included"""
        self.client.force_authenticate(self.frank)
        response = self.client.get(reverse("user-list"))
        user_names = sorted(user["username"] for user in response.json())
        self.assertIn("cern-fsauerbu", user_names)

    def test_users_list_self_internal(self):
        """Check that one's own internal profile is included"""
        self.client.force_authenticate(self.karl)
        response = self.client.get(reverse("user-list"))
        user_names = sorted(user["username"] for user in response.json())
        self.assertEqual(user_names, ["cern-fsauerbu", "cern-karl"])

    def test_users_list_self_private(self):
        """Check that one's own private profile is included"""
        self.client.force_authenticate(self.john)
        response = self.client.get(reverse("user-list"))
        user_names = sorted(user["username"] for user in response.json())
        self.assertEqual(user_names,
                         ["cern-fsauerbu", "cern-john", "cern-karl"])

    ###########################################################
    # User details
    def test_users_details_info(self):
        """Check that the correct information is included"""
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))
        info = response.json()
        self.assertEqual(info["username"], "cern-fsauerbu")
        self.assertEqual(info["first_name"], "Frank")
        self.assertEqual(info["last_name"], "Sauerburger")
        self.assertIn("collections", info)

    def test_users_details_public_anon(self):
        """Check that anonymous users can read public profiles"""
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()["first_name"], "Frank")

    def test_users_details_public_auth(self):
        """Check that authorized users can read public profiles"""
        self.client.force_authenticate(self.john)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()["first_name"], "Frank")
        self.assertEqual(response.json()["visibility"], PUBLIC_LEVEL)

    def test_users_details_public_auth_self(self):
        """Check that one self can read public profiles"""
        self.client.force_authenticate(self.frank)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()["first_name"], "Frank")

    def test_users_details_internal_anon(self):
        """Check that anonymous users cannot access internal profiles"""
        response = self.client.get(reverse("user-detail",
                                           args=["cern-karl"]))

        self.assertEqual(response.status_code, 404)

    def test_users_details_internal_auth(self):
        """Check that authenticated users can access internal profiles"""
        self.client.force_authenticate(self.frank)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-karl"]))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()["first_name"], "Karl")
        self.assertEqual(response.json()["visibility"], INTERNAL_LEVEL)

    def test_users_details_internal_auth_self(self):
        """Check that one self can access the internal profile"""
        self.client.force_authenticate(self.karl)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-karl"]))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()["first_name"], "Karl")

    def test_users_details_private_anon(self):
        """Check that anonymous users cannot access private profiles"""
        pass
        response = self.client.get(reverse("user-detail",
                                           args=["cern-john"]))

        self.assertEqual(response.status_code, 404)

    def test_users_details_private_auth(self):
        """Check that authenticated users cannot access private profiles"""
        self.client.force_authenticate(self.frank)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-john"]))

        self.assertEqual(response.status_code, 404)

    def test_users_details_private_auth_self(self):
        """Check that one self can access the private profile"""
        self.client.force_authenticate(self.john)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-john"]))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json()["first_name"], "John")
        self.assertEqual(response.json()["visibility"], PRIVATE_LEVEL)

    ###########################################################
    # Create, update and delete
    def test_post_anon(self):
        """Check that anonymous posting fails"""
        response = self.client.post(reverse("user-list"))
        self.assertEqual(response.status_code, 405)

    def test_post_auth(self):
        """Check that authorized posting fails"""
        self.client.force_authenticate(self.john)
        response = self.client.post(reverse("user-list"))
        self.assertEqual(response.status_code, 405)

    def test_put_anon(self):
        """Check that anonymous updates fails"""
        response = self.client.put(reverse("user-detail",
                                           args=["cern-john"]))
        self.assertEqual(response.status_code, 405)

    def test_put_auth(self):
        """Check that authorized updates fails"""
        self.client.force_authenticate(self.frank)
        response = self.client.put(reverse("user-detail",
                                           args=["cern-john"]))
        self.assertEqual(response.status_code, 405)

    def test_put_auth_self(self):
        """Check that authorized updates to one self fails"""
        self.client.force_authenticate(self.john)
        response = self.client.put(reverse("user-detail",
                                           args=["cern-john"]))
        self.assertEqual(response.status_code, 405)

    def test_delete_anon(self):
        """Check that anonymous deletions fails"""
        response = self.client.delete(
            reverse("user-detail", args=["cern-john"])
        )
        self.assertEqual(response.status_code, 405)

    def test_delete_auth(self):
        """Check that authorized deletions fails"""
        self.client.force_authenticate(self.frank)
        response = self.client.delete(
            reverse("user-detail", args=["cern-john"])
        )
        self.assertEqual(response.status_code, 405)

    def test_delete_auth_self(self):
        """Check that authorized deletions to one self fails"""
        self.client.force_authenticate(self.john)
        response = self.client.delete(
            reverse("user-detail", args=["cern-john"])
        )
        self.assertEqual(response.status_code, 405)

    ###########################################################
    # Related collections at /user
    def test_users_related_anon__public_user(self):
        """Check that the user endpoint shows public related collections"""
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        # Follow API links
        urls = [url for url in response.json()["collections"]]
        collections = [self.client.get(url).json() for url in urls]
        titles = [collection["title"] for collection in collections]
        titles.sort()

        self.assertEqual(titles, ["Frank at the zoo"])

    def test_users_related_auth__public_user(self):
        """Check that the user endpoint shows internal related collections"""
        self.client.force_authenticate(self.karl)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        # Follow API links
        urls = [url for url in response.json()["collections"]]
        collections = [self.client.get(url).json() for url in urls]
        titles = [collection["title"] for collection in collections]
        titles.sort()

        self.assertEqual(titles, ["Frank at the zoo", "Frank in the office"])

    def test_users_related_auth_self__public_user(self):
        """Check that the user endpoint includes own private collection"""
        self.client.force_authenticate(self.frank)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        # Follow API links
        urls = [url for url in response.json()["collections"]]
        collections = [self.client.get(url).json() for url in urls]
        titles = [collection["title"] for collection in collections]
        titles.sort()

        self.assertEqual(titles, ["Frank at home",
                                  "Frank at the zoo",
                                  "Frank in the office"])

    def test_users_related_auth__internal_user(self):
        """Check that the user endpoint shows internal related collections"""
        self.client.force_authenticate(self.john)
        response = self.client.get(reverse("user-detail",
                                           args=["cern-karl"]))

        # Follow API links
        urls = [url for url in response.json()["collections"]]
        collections = [self.client.get(url).json() for url in urls]
        titles = [collection["title"] for collection in collections]
        titles.sort()

        self.assertEqual(titles, ["Karl at the zoo", "Karl in the office"])

    def test_users_detail_headers(self):
        """Check that detail response contains the correct headers"""
        response = self.client.get(reverse("user-detail",
                                           args=["cern-fsauerbu"]))

        self.assertIn("cern-fsauerbu", response["link"])
        self.assertIn("ui", response["link"])
